#% Ej 19

import numpy as np

import random
import plotly.express as px
import matplotlib.pyplot as plt
import scipy as sc

from IPython import get_ipython

get_ipython().run_line_magic('matplotlib', 'inline')

#%%

N = sc.stats.norm.cdf

def Black_Scholes_Call(S, K, T, r, sigma):
    d1 = (np.log(S/K) + (r + sigma**2/2)*T) / (sigma*np.sqrt(T))
    d2 = d1 - sigma * np.sqrt(T)
    return S * N(d1) - K * np.exp(-r*T)* N(d2)

def Black_Scholes_Put(S, K, T, r, sigma):
    d1 = (np.log(S/K) + (r + sigma**2/2)*T) / (sigma*np.sqrt(T))
    d2 = d1 - sigma* np.sqrt(T)
    return K*np.exp(-r*T)*N(-d2) - S*N(-d1)

#%%
    
K = np.linspace(4000,6000,1000)

BS = Black_Scholes_Call(4930, K ,1/12, 0.01, 0.2)

fig = plt.figure()
fig.set_figheight(8)
fig.set_figwidth(12)
plt.plot(K,BS),plt.show()

#%% 
K = np.linspace(1,6000,1000)
BS = Black_Scholes_Call(K, K ,1/12, 0.01, 0.2)

fig = plt.figure()
fig.set_figheight(8)
fig.set_figwidth(12)
plt.plot(K,BS),plt.show()

#%% Simulation

N_samples= 100000
K = 4930
S_0 = K
T= 1/12
r =0.01
sigma= 0.2

S_t = S_0*np.exp(sigma*np.array([np.random.randn()*np.sqrt(T) for i in range(N_samples)])  + (r - sigma**2/2)*T )

fun = np.zeros_like(S_t)
for i in range(N_samples):
    if S_t[i] -K > 0:
        fun[i] = S_t[i] -K

E_St_K = np.mean(fun)

Call_BS = np.exp(-r*T)*E_St_K

print('Simulation: ',Call_BS)
print('B_S_formula: ', Black_Scholes_Call(S_0, K, T, r, sigma))


#%% Evolution in time

time = np.linspace(0,10,120)
K = 4930
S_0 = 6000
T= 1/12
r =0.01
sigma= 0.2

BS = Black_Scholes_Call(S_0, S_0 ,time, r, sigma)

fig = plt.figure()
fig.set_figheight(8)
fig.set_figwidth(12)
plt.plot(time,BS),plt.show()
